@@ -27,7 +27,6 @@
 
 G_BEGIN_DECLS
 
-typedef struct _GIrTypelibBuild GIrTypelibBuild;
 typedef struct _GIrNode GIrNode;
 typedef struct _GIrNodeFunction GIrNodeFunction;
 typedef struct _GIrNodeParam GIrNodeParam;
@@ -46,16 +45,6 @@ typedef struct _GIrNodeErrorDomain GIrNodeErrorDomain;
 typedef struct _GIrNodeXRef GIrNodeXRef;
 typedef struct _GIrNodeUnion GIrNodeUnion;
 
-struct _GIrTypelibBuild {
-  GIrModule  *module;
-  GList       *modules;
-  GHashTable  *strings;
-  GHashTable  *types;
-  GList       *nodes_with_attributes;
-  guint32      n_attributes;
-  guchar      *data;
-};
-
 typedef enum
 {
   G_IR_NODE_INVALID      =  0,
@@ -84,6 +73,7 @@ struct _GIrNode
 {
   GIrNodeTypeId type;
   gchar *name;
+  GIrModule *module;
 
   guint32 offset; /* Assigned as we build the typelib */
 
@@ -244,10 +234,16 @@ struct _GIrNodeInterface
 
   gboolean abstract;
   gboolean deprecated;
+  gboolean fundamental;
 
   gchar *gtype_name;
   gchar *gtype_init;
 
+  gchar *ref_func;
+  gchar *unref_func;
+  gchar *set_value_func;
+  gchar *get_value_func;
+
   gchar *parent;
   gchar *glib_type_struct;
 
@@ -266,7 +262,7 @@ struct _GIrNodeValue
 
   gboolean deprecated;
 
-  gint32 value;
+  gint64 value;
 };
 
 struct _GIrNodeConstant
@@ -358,39 +354,37 @@ struct _GIrNodeErrorDomain
 };
 
 
-GIrNode * g_ir_node_new             (GIrNodeTypeId type);
-void      g_ir_node_free            (GIrNode    *node);
-guint32   g_ir_node_get_size        (GIrNode    *node);
-guint32   g_ir_node_get_full_size   (GIrNode    *node);
-void      g_ir_node_build_typelib   (GIrNode         *node,
-                                     GIrNode         *parent,
-                                     GIrTypelibBuild *build,
-                                     guint32         *offset,
-                                     guint32         *offset2);
-int       g_ir_node_cmp             (GIrNode    *node,
+GIrNode * _g_ir_node_new             (GIrNodeTypeId type,
+				     GIrModule     *module);
+void      _g_ir_node_free            (GIrNode    *node);
+guint32   _g_ir_node_get_size        (GIrNode    *node);
+guint32   _g_ir_node_get_full_size   (GIrNode    *node);
+void      _g_ir_node_build_typelib   (GIrNode         *node,
+				      GIrNode         *parent,
+				      GIrTypelibBuild *build,
+				      guint32         *offset,
+				      guint32         *offset2);
+int       _g_ir_node_cmp             (GIrNode    *node,
 				     GIrNode    *other);
-gboolean  g_ir_node_can_have_member (GIrNode    *node);
-void      g_ir_node_add_member      (GIrNode         *node,
-				     GIrNodeFunction *member);
-guint32   write_string              (const gchar *str,
-				     GHashTable  *strings,
-				     guchar      *data,
-				     guint32     *offset);
+gboolean  _g_ir_node_can_have_member (GIrNode    *node);
+void      _g_ir_node_add_member      (GIrNode         *node,
+				      GIrNodeFunction *member);
+guint32   _g_ir_write_string              (const gchar *str,
+					   GHashTable  *strings,
+					   guchar      *data,
+					   guint32     *offset);
 
-const gchar * g_ir_node_param_direction_string (GIrNodeParam * node);
-const gchar * g_ir_node_type_to_string         (GIrNodeTypeId type);
+const gchar * _g_ir_node_param_direction_string (GIrNodeParam * node);
+const gchar * _g_ir_node_type_to_string         (GIrNodeTypeId type);
 
-gboolean g_ir_find_node (GIrModule  *module,
-			 GList      *modules,
-			 const char *name,
-			 GIrNode   **node_out,
-			 GIrModule **module_out);
+GIrNode *_g_ir_find_node (GIrTypelibBuild  *build,
+			 GIrModule        *module,
+			 const char       *name);
 
 /* In giroffsets.c */
 
-void g_ir_node_compute_offsets (GIrNode   *node,
-			        GIrModule *module,
-			        GList     *modules);
+void _g_ir_node_compute_offsets (GIrTypelibBuild *build,
+				GIrNode         *node);
 
 
 G_END_DECLS
