@@ -18,13 +18,14 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include "girepository-private.h"
+#include "girffi.h"
 #include "girnode.h"
+#include <string.h>
 
 /* The C standard specifies that an enumeration can be any char or any signed
- * or unsigned integer type capable of resresenting all the values of the
+ * or unsigned integer type capable of representing all the values of the
  * enumeration. We use test enumerations to figure out what choices the
- * compiler makes.
+ * compiler makes. (Ignoring > 32 bit enumerations)
  */
 
 typedef enum {
@@ -51,24 +52,26 @@ typedef enum {
   ENUM_6 = ((guint)G_MAXINT) + 1 /* compiler could use uint32 */
 } Enum6;
 
-/* GIrNodeValue has guint32 values, so if it matters to the ABI whether
- * constant values are signed, we are in trouble. And we don't handle
- * enums with > 32 bit values. */
-
-#if 0
 typedef enum {
   ENUM_7 = -1 /* compiler could use int8, int16, int32 */
 } Enum7;
 
-/* etc... */
-#endif
+typedef enum {
+  ENUM_8 = -129 /* compiler could use int16, int32 */
+} Enum8;
+
+typedef enum {
+  ENUM_9 = G_MINSHORT - 1 /* compiler could use int32 */
+} Enum9;
 
 static void
 compute_enum_storage_type (GIrNodeEnum *enum_node)
 {
   GList *l;
-  gint32 max_value = 0;
+  gint64 max_value = 0;
+  gint64 min_value = 0;
   int width;
+  gboolean signed_type;
 
   if (enum_node->storage_type != GI_TYPE_TAG_VOID) /* already done */
     return;
@@ -78,29 +81,63 @@ compute_enum_storage_type (GIrNodeEnum *enum_node)
       GIrNodeValue *value = l->data;
       if (value->value > max_value)
         max_value = value->value;
+      if (value->value < min_value)
+        min_value = value->value;
     }
 
-  if (max_value < 128)
-    width = sizeof (Enum1);
-  else if (max_value < 256)
-    width = sizeof (Enum2);
-  else if (max_value < G_MAXSHORT)
-    width = sizeof (Enum3);
-  else if (max_value < G_MAXUSHORT)
-    width = sizeof (Enum4);
-  else if (max_value < G_MAXINT)
-    width = sizeof (Enum5);
+  if (min_value < 0)
+    {
+      signed_type = TRUE;
+
+      if (min_value > -128 && max_value <= 127)
+	width = sizeof(Enum7);
+      else if (min_value >= G_MINSHORT && max_value <= G_MAXSHORT)
+	width = sizeof(Enum8);
+      else
+	width = sizeof(Enum9);
+    }
   else
-    width = sizeof (Enum6);
+    {
+      if (max_value <= 127)
+	{
+	  width = sizeof (Enum1);
+	  signed_type = (gint64)(Enum1)(-1) < 0;
+	}
+      else if (max_value <= 255)
+	{
+	  width = sizeof (Enum2);
+	  signed_type = (gint64)(Enum2)(-1) < 0;
+	}
+      else if (max_value <= G_MAXSHORT)
+	{
+	  width = sizeof (Enum3);
+	  signed_type = (gint64)(Enum3)(-1) < 0;
+	}
+      else if (max_value <= G_MAXUSHORT)
+	{
+	  width = sizeof (Enum4);
+	  signed_type = (gint64)(Enum4)(-1) < 0;
+	}
+      else if (max_value <= G_MAXINT)
+	{
+	  width = sizeof (Enum5);
+	  signed_type = (gint64)(Enum5)(-1) < 0;
+	}
+      else
+	{
+	  width = sizeof (Enum6);
+	  signed_type = (gint64)(Enum6)(-1) < 0;
+	}
+    }
 
   if (width == 1)
-    enum_node->storage_type = GI_TYPE_TAG_UINT8;
+    enum_node->storage_type = signed_type ? GI_TYPE_TAG_INT8 : GI_TYPE_TAG_UINT8;
   else if (width == 2)
-    enum_node->storage_type = GI_TYPE_TAG_UINT16;
+    enum_node->storage_type = signed_type ? GI_TYPE_TAG_INT16 : GI_TYPE_TAG_UINT16;
   else if (width == 4)
-    enum_node->storage_type = GI_TYPE_TAG_UINT32;
+    enum_node->storage_type = signed_type ? GI_TYPE_TAG_INT32 : GI_TYPE_TAG_UINT32;
   else if (width == 8)
-    enum_node->storage_type = GI_TYPE_TAG_UINT64;
+    enum_node->storage_type = signed_type ? GI_TYPE_TAG_INT64 : GI_TYPE_TAG_UINT64;
   else
     g_error ("Unexpected enum width %d", width);
 }
@@ -144,26 +181,24 @@ get_enum_size_alignment (GIrNodeEnum *enum_node,
 }
 
 static gboolean
-get_interface_size_alignment (GIrNodeType *type,
-                              GIrModule   *module,
-			      GList       *modules,
+get_interface_size_alignment (GIrTypelibBuild   *build,
+			      GIrNodeType *type,
 			      gint        *size,
 			      gint        *alignment,
                               const char  *who)
 {
   GIrNode *iface;
-  GIrModule *iface_module;
 
-  if (!g_ir_find_node (module, modules, type->interface, &iface, &iface_module))
+  iface = _g_ir_find_node (build, ((GIrNode*)type)->module, type->interface);
+  if (!iface)
     {
-      g_ir_module_fatal (module, 0, "Can't resolve type '%s' for %s", type->interface, who);
+      _g_ir_module_fatal (build, 0, "Can't resolve type '%s' for %s", type->interface, who);
       *size = -1;
       *alignment = -1;
       return FALSE;
     }
 
-  g_ir_node_compute_offsets (iface, iface_module,
-			     iface_module == module ? modules : NULL);
+  _g_ir_node_compute_offsets (build, iface);
 
   switch (iface->type)
     {
@@ -212,7 +247,7 @@ get_interface_size_alignment (GIrNodeType *type,
       {
 	g_warning ("%s has is not a pointer and is of type %s",
                    who,
-		   g_ir_node_type_to_string (iface->type));
+		   _g_ir_node_type_to_string (iface->type));
 	*size = -1;
 	*alignment = -1;
 	break;
@@ -223,9 +258,8 @@ get_interface_size_alignment (GIrNodeType *type,
 }
 
 static gboolean
-get_type_size_alignment (GIrNodeType *type,
-                         GIrModule   *module,
-                         GList       *modules,
+get_type_size_alignment (GIrTypelibBuild   *build,
+			 GIrNodeType *type,
                          gint        *size,
                          gint        *alignment,
                          const char  *who)
@@ -241,7 +275,7 @@ get_type_size_alignment (GIrNodeType *type,
       gint elt_size, elt_alignment;
 
       if (!type->has_size
-          || !get_type_size_alignment(type->parameter_type1, module, modules,
+          || !get_type_size_alignment(build, type->parameter_type1,
                                       &elt_size, &elt_alignment, who))
         {
           *size = -1;
@@ -258,11 +292,11 @@ get_type_size_alignment (GIrNodeType *type,
     {
       if (type->tag == GI_TYPE_TAG_INTERFACE)
 	{
-	  return get_interface_size_alignment (type, module, modules, size, alignment, who);
+	  return get_interface_size_alignment (build, type, size, alignment, who);
 	}
       else
 	{
-	  type_ffi = _gi_type_tag_get_ffi_type (type->tag, type->is_pointer);
+	  type_ffi = gi_type_tag_get_ffi_type (type->tag, type->is_pointer);
 
 	  if (type_ffi == &ffi_type_void)
 	    {
@@ -291,13 +325,13 @@ get_type_size_alignment (GIrNodeType *type,
 }
 
 static gboolean
-get_field_size_alignment (GIrNodeField *field,
+get_field_size_alignment (GIrTypelibBuild    *build,
+			  GIrNodeField *field,
 			  GIrNode      *parent_node,
-			  GIrModule    *module,
-			  GList        *modules,
 			  gint         *size,
 			  gint         *alignment)
 {
+  GIrModule *module = build->module;
   gchar *who;
   gboolean success;
 
@@ -310,7 +344,7 @@ get_field_size_alignment (GIrNodeField *field,
       success = TRUE;
     }
   else
-    success = get_type_size_alignment (field->type, module, modules, size, alignment, who);
+    success = get_type_size_alignment (build, field->type, size, alignment, who);
   g_free (who);
 
   return success;
@@ -319,10 +353,9 @@ get_field_size_alignment (GIrNodeField *field,
 #define ALIGN(n, align) (((n) + (align) - 1) & ~((align) - 1))
 
 static gboolean
-compute_struct_field_offsets (GIrNode     *node,
+compute_struct_field_offsets (GIrTypelibBuild   *build,
+			      GIrNode     *node,
 			      GList       *members,
-			      GIrModule   *module,
-			      GList       *modules,
 			      gint        *size_out,
 			      gint        *alignment_out)
 {
@@ -346,8 +379,7 @@ compute_struct_field_offsets (GIrNode     *node,
 	      int member_size;
 	      int member_alignment;
 
-	      if (get_field_size_alignment (field, node,
-					    module, modules,
+	      if (get_field_size_alignment (build, field, node,
 					    &member_size, &member_alignment))
 		{
 		  size = ALIGN (size, member_alignment);
@@ -388,10 +420,9 @@ compute_struct_field_offsets (GIrNode     *node,
 }
 
 static gboolean
-compute_union_field_offsets (GIrNode     *node,
+compute_union_field_offsets (GIrTypelibBuild *build,
+			     GIrNode     *node,
 			     GList       *members,
-			     GIrModule   *module,
-			     GList       *modules,
 			     gint        *size_out,
 			     gint        *alignment_out)
 {
@@ -415,8 +446,7 @@ compute_union_field_offsets (GIrNode     *node,
 	      int member_size;
 	      int member_alignment;
 
-	      if (get_field_size_alignment (field, node,
-					    module, modules,
+	      if (get_field_size_alignment (build,field, node,
 					    &member_size, &member_alignment))
 		{
 		  size = MAX (size, member_size);
@@ -446,10 +476,11 @@ compute_union_field_offsets (GIrNode     *node,
 }
 
 static gboolean
-check_needs_computation (GIrNode   *node,
-			 GIrModule *module,
+check_needs_computation (GIrTypelibBuild *build,
+			 GIrNode   *node,
 			 gint       alignment)
 {
+  GIrModule *module = build->module;
   /*
    *  0: Not yet computed
    * >0: Previously succeeded
@@ -466,31 +497,37 @@ check_needs_computation (GIrNode   *node,
 }
 
 /**
- * g_ir_node_compute_offsets:
+ * _g_ir_node_compute_offsets:
+ * @build: Current typelib build
  * @node: a #GIrNode
- * @module: Current module being processed
- * @modules: all currently loaded modules
  *
  * If a node is a a structure or union, makes sure that the field
  * offsets have been computed, and also computes the overall size and
  * alignment for the type.
  */
 void
-g_ir_node_compute_offsets (GIrNode   *node,
-			   GIrModule *module,
-			   GList     *modules)
+_g_ir_node_compute_offsets (GIrTypelibBuild *build,
+			    GIrNode         *node)
 {
+  gboolean appended_stack;
+
+  if (build->stack)
+    appended_stack = node != (GIrNode*)build->stack->data; 
+  else
+    appended_stack = TRUE;
+  if (appended_stack)
+    build->stack = g_list_prepend (build->stack, node);
+
   switch (node->type)
     {
     case G_IR_NODE_BOXED:
       {
 	GIrNodeBoxed *boxed = (GIrNodeBoxed *)node;
 
-	if (!check_needs_computation (node, module, boxed->alignment))
+	if (!check_needs_computation (build, node, boxed->alignment))
 	  return;
 
-	compute_struct_field_offsets (node, boxed->members,
-				      module, modules,
+	compute_struct_field_offsets (build, node, boxed->members,
 				      &boxed->size, &boxed->alignment);
 	break;
       }
@@ -498,11 +535,10 @@ g_ir_node_compute_offsets (GIrNode   *node,
       {
 	GIrNodeStruct *struct_ = (GIrNodeStruct *)node;
 
-	if (!check_needs_computation (node, module, struct_->alignment))
+	if (!check_needs_computation (build, node, struct_->alignment))
 	  return;
 
-	compute_struct_field_offsets (node, struct_->members,
-				      module, modules,
+	compute_struct_field_offsets (build, node, struct_->members,
 				      &struct_->size, &struct_->alignment);
 	break;
       }
@@ -511,11 +547,10 @@ g_ir_node_compute_offsets (GIrNode   *node,
       {
 	GIrNodeInterface *iface = (GIrNodeInterface *)node;
 
-	if (!check_needs_computation (node, module, iface->alignment))
+	if (!check_needs_computation (build, node, iface->alignment))
 	  return;
 
-	compute_struct_field_offsets (node, iface->members,
-				      module, modules,
+	compute_struct_field_offsets (build, node, iface->members,
 				      &iface->size, &iface->alignment);
 	break;
       }
@@ -523,11 +558,10 @@ g_ir_node_compute_offsets (GIrNode   *node,
       {
 	GIrNodeUnion *union_ = (GIrNodeUnion *)node;
 
-	if (!check_needs_computation (node, module, union_->alignment))
+	if (!check_needs_computation (build, node, union_->alignment))
 	  return;
 
-	compute_union_field_offsets (node, union_->members,
-				     module, modules,
+	compute_union_field_offsets (build, (GIrNode*)union_, union_->members,
 				     &union_->size, &union_->alignment);
 	break;
       }
@@ -544,7 +578,9 @@ g_ir_node_compute_offsets (GIrNode   *node,
 	break;
       }
     default:
-      /* Nothing to do */
-      return;
+      break;
     }
+  
+  if (appended_stack)
+    build->stack = g_list_delete_link (build->stack, build->stack);
 }
