@@ -53,16 +53,6 @@ class MainTransformer(object):
 * Not including .h files to be scanned
 * Broken --identifier-prefix
 """)
-        ## WORKAROUND ##
-        # Dirty hack for now...maybe eventually we'll support the "typedef GSList FooSet"
-        # pattern.
-        # https://bugzilla.gnome.org/show_bug.cgi?id=629682
-        if self._namespace.name == 'Atk':
-            attribute = self._namespace.get('Attribute')
-            attributeset = self._namespace.get('AttributeSet')
-            if attribute and attributeset:
-                alias = ast.Alias('AttributeSet', target=ast.TYPE_ANY)
-                self._namespace.append(alias, replace=True)
 
         # Some initial namespace surgery
         self._namespace.walk(self._pass_fixup_hidden_fields)
@@ -510,7 +500,10 @@ usage is void (*_gtk_reserved1)(void);"""
             annotated_direction = ast.PARAM_DIRECTION_OUT
             if subtype in (None, ''):
                 if node.type.target_giname and node.type.ctype:
-                    caller_allocates = '**' not in node.type.ctype
+                    target = self._transformer.lookup_giname(node.type.target_giname)
+                    has_double_indirection = '**' in node.type.ctype
+                    is_structure_or_union = isinstance(target, (ast.Record, ast.Union))
+                    caller_allocates = (not has_double_indirection and is_structure_or_union)
                 else:
                     caller_allocates = False
             elif subtype == OPT_OUT_CALLER_ALLOCATES:
@@ -716,12 +709,15 @@ usage is void (*_gtk_reserved1)(void);"""
         # than the number of signal parameters
         if block and len(block.tags) > len(signal.parameters):
             names = block.tags.items()
+            # Resolve real parameter names early, so that in later
+            # phase we can refer to them while resolving annotations.
+            for i, param in enumerate(signal.parameters):
+                param.argname, tag = names[i+1]
         else:
             names = []
         for i, param in enumerate(signal.parameters):
             if names:
                 name, tag = names[i+1]
-                param.name = name
                 options = getattr(tag, 'options', {})
                 param_type = options.get(OPT_TYPE)
                 if param_type:
@@ -912,16 +908,29 @@ method or constructor of some type."""
 
         # A quick hack here...in the future we should catch C signature/GI signature
         # mismatches in a general way in finaltransformer
-        if first.type.ctype.count('*') != 1:
+        if first.type.ctype is not None and first.type.ctype.count('*') != 1:
             return False
 
-        uscored = self._uscored_identifier_for_type(first.type)
-        if not subsymbol.startswith(uscored):
-            return False
+        # Here we check both the c_symbol_prefix and (if that fails),
+        # attempt to do a default uscoring of the type.  The reason we
+        # look at a default underscore transformation is for
+        # gdk_window_object_get_type(), which says to us that the
+        # prefix is "gdk_window_object", when really it's just
+        # "gdk_window".  Possibly need an annotation to override this.
+        prefix_matches = False
+        uscored_prefix = None
+        if hasattr(target, 'c_symbol_prefix') and target.c_symbol_prefix is not None:
+            prefix_matches = subsymbol.startswith(target.c_symbol_prefix)
+            if prefix_matches:
+                uscored_prefix = target.c_symbol_prefix
+        if not prefix_matches:
+            uscored_prefix = self._uscored_identifier_for_type(first.type)
+            if not subsymbol.startswith(uscored_prefix):
+                return False
         func.instance_parameter = func.parameters.pop(0)
         subsym_idx = func.symbol.find(subsymbol)
         self._namespace.float(func)
-        func.name = func.symbol[(subsym_idx + len(uscored) + 1):]
+        func.name = func.symbol[(subsym_idx + len(uscored_prefix) + 1):]
         target.methods.append(func)
         func.is_method = True
         return True
@@ -1026,10 +1035,6 @@ method or constructor of some type."""
             if isinstance(field, ast.Field):
                 field.writable = False
 
-        # Loop through fields to determine which are virtual
-        # functions and which are signal slots by
-        # assuming everything that doesn't share a name
-        # with a known signal is a virtual slot.
         for field in class_struct.fields:
             if not isinstance(field.anonymous_node, ast.Callback):
                 continue
@@ -1040,15 +1045,6 @@ method or constructor of some type."""
             firstparam_type = callback.parameters[0].type
             if firstparam_type != node_type:
                 continue
-            # Also double check we don't have a signal with this
-            # name.
-            matched_signal = False
-            for signal in node.signals:
-                if signal.name.replace('-', '_') == callback.name:
-                    matched_signal = True
-                    break
-            if matched_signal:
-                continue
             vfunc = ast.VFunction.from_callback(callback)
             vfunc.instance_parameter = callback.parameters[0]
             vfunc.inherit_file_positions(callback)
