@@ -45,7 +45,7 @@ gboolean include_cwd = FALSE;
 gboolean debug = FALSE;
 gboolean verbose = FALSE;
 
-static void
+static gboolean
 write_out_typelib (gchar *prefix,
 		   GITypelib *typelib)
 {
@@ -56,6 +56,7 @@ write_out_typelib (gchar *prefix,
   GFile *tmp_file_obj;
   gchar *tmp_filename;
   GError *error = NULL;
+  gboolean success = FALSE;
 
   if (output == NULL)
     {
@@ -81,16 +82,16 @@ write_out_typelib (gchar *prefix,
 
       if (file == NULL)
 	{
-	  g_fprintf (stderr, "failed to open '%s': %s\n",
-		     tmp_filename, g_strerror (errno));
-	  goto out;
+          g_fprintf (stderr, "failed to open '%s': %s\n",
+                     tmp_filename, g_strerror (errno));
+          goto out;
 	}
     }
 
   written = fwrite (typelib->data, 1, typelib->len, file);
   if (written < typelib->len) {
-    g_error ("ERROR: Could not write the whole output: %s",
-	     strerror(errno));
+    g_fprintf (stderr, "ERROR: Could not write the whole output: %s",
+	       strerror(errno));
     goto out;
   }
 
@@ -100,13 +101,17 @@ write_out_typelib (gchar *prefix,
     {
       if (!g_file_move (tmp_file_obj, file_obj, G_FILE_COPY_OVERWRITE, NULL, NULL, NULL, &error))
         {
-          g_error ("ERROR: failed to rename %s to %s: %s", tmp_filename, filename, error->message);
+	  g_fprintf (stderr, "ERROR: failed to rename %s to %s: %s", tmp_filename, filename, error->message);
           g_clear_error (&error);
+	  goto out;
         }
     }
+  success = TRUE;
 out:
   g_free (filename);
   g_free (tmp_filename);
+
+  return success;
 }
 
 GLogLevelFlags logged_levels;
@@ -174,11 +179,11 @@ main (int argc, char ** argv)
     for (i = 0; includedirs[i]; i++)
       g_irepository_prepend_search_path (includedirs[i]);
 
-  parser = g_ir_parser_new ();
+  parser = _g_ir_parser_new ();
 
-  g_ir_parser_set_includes (parser, (const char*const*) includedirs);
+  _g_ir_parser_set_includes (parser, (const char*const*) includedirs);
 
-  module = g_ir_parser_parse_file (parser, input[0], &error);
+  module = _g_ir_parser_parse_file (parser, input[0], &error);
   if (module == NULL) 
     {
       g_fprintf (stderr, "error parsing file %s: %s\n", 
@@ -203,14 +208,15 @@ main (int argc, char ** argv)
 
       g_debug ("[building] module %s", module->name);
 
-      typelib = g_ir_module_build_typelib (module);
+      typelib = _g_ir_module_build_typelib (module);
       if (typelib == NULL)
 	g_error ("Failed to build typelib for module '%s'\n", module->name);
       if (!g_typelib_validate (typelib, &error))
 	g_error ("Invalid typelib for module '%s': %s", 
 		 module->name, error->message);
 
-      write_out_typelib (NULL, typelib);
+      if (!write_out_typelib (NULL, typelib))
+	return 1;
       g_typelib_free (typelib);
       typelib = NULL;
     }
@@ -219,7 +225,7 @@ main (int argc, char ** argv)
 
 #if 0
   /* No point */
-  g_ir_parser_free (parser);
+  _g_ir_parser_free (parser);
 #endif  
 
   return 0; 
